# -*- python -*-
# $RCSfile: gfxmanager.py,v $
# $Revision: 1.17 $
# $Author: reida $
# $Date: 2005/08/18 20:43:36 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

# The GfxWindowManager should probably be replaced with a set of
# switchboard callbacks.  There's not really any need for it any more. 

from oof2.SWIG.common import switchboard
from oof2.common import debug
from oof2.common import utils
from oof2.common import guitop
import sys

class GfxWindowManager:
    """
    The GfxWindowManager keeps track of all the open graphics windows
    associated with a top level GUI window.  The top level window
    creates graphics windows by talking to its manager.
    """
    def __init__(self):
        self.windows = []
        self.count = 0
    def openWindow(self, **kwargs):
        self.count += 1
        count = self.count
        name = 'Graphics_' + `count`
        w = self._newWindow(name, count, **kwargs) # opens real or ghost gfx window
        self.windows.append(w)
        switchboard.notify('open graphics window', w)
        return w
    
    def _newWindow(self, name, count, **kwargs):
        # This function is redefined if the GUI is loaded.
        from oof2.common.IO import ghostgfxwindow
        return ghostgfxwindow.GhostGfxWindow(name, count, self, **kwargs)
    def closeWindow(self, w):
        # Tell the graphics window that it's been closed, so it
        # doesn't try to repeat this call when it gets its gtk.destroy
        # signal.  You can't just not make the call in that case,
        # because gtk.destroy can be sent either by the window
        # manager, when the window is closed, or at exit-time.
        w.closed = 1
        self.windows.remove(w)
        switchboard.notify('close graphics window', w)
    def apply(self, func, *args):
        for window in self.windows:
            apply(func, (window,)+args)
    def getWindow(self, name):
        for window in self.windows:
            if window.name == name:
                return window
        raise KeyError("No such window: " + `name`)
    def getAllWindows(self):
        return self.windows[:]

gfxManager = GfxWindowManager()

