// -*- C++ -*-
// $RCSfile: stringimage.h,v $
// $Revision: 1.8 $
// $Author: langer $
// $Date: 2005/08/08 21:39:28 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

// Class for representing an Image as a string, as a step on the way
// to creating a GdkImlibImage.  Putting this class in C++ instead of
// Python allows the C++ BitmapOverlay to be drawn without passing it
// back out to Python.

#ifndef STRINGIMAGE
#define STRINGIMAGE

#include "common/coord.h"
#include "common/ccolor.h"
#include <string>

class BitmapOverlay;
class BoolArray;
class ShapedStringImage;

class StringImage {
protected:
  unsigned char *data;
  ICoord isize_;
  Coord size_;
  int getOffset(const ICoord&) const;
public:
  StringImage(const ICoord *isize,  const Coord *size);
  ~StringImage();
  const ICoord &sizeInPixels() const { return isize_; }
  const Coord &size() const { return size_; }
  void set(const ICoord *where, const CColor *color);
  CColor get(const ICoord *) const;
  ShapedStringImage *blend(const BitmapOverlay*, double opacity) const;
  ShapedStringImage *mask(const BoolArray &bits) const;
  const unsigned char *getString() const { return data; }
  //used for post script output
  std::string hexstringimage() const;
  std::string hexstringmask(CColor *) const;
};

class CharColor {
  // Used internally in ShapedString Image to avoid converting between
  // floating point and int representations of colors.
public:
  CharColor(unsigned char *rgb)
    : r(*rgb), g(*(rgb+1)), b(*rgb+2) {}
  CharColor(unsigned char r, unsigned char g, unsigned char b)
    : r(r), g(g), b(b) {}
  CharColor() : r(0), g(0), b(0) {}
  unsigned char r, g, b;
  CColor ccolor() const { return CColor(r/255., g/255., b/255.); }
};
std::ostream &operator<<(std::ostream&, const CharColor&);


class ShapedStringImage: public StringImage {
private:
  void findBGcolor(const BoolArray&);
  void applyMask(const BoolArray&);
  CharColor shapecolor_;
public:
  ShapedStringImage(const StringImage&, const BitmapOverlay&, double opacity);
  ShapedStringImage(const StringImage&, const BoolArray&);
  CColor getBG() const;
};

//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//

class AlphaStringImage {
protected:
  unsigned char *data;
  ICoord isize_;
  Coord size_;
  int getOffset(const ICoord &) const;
public:
  AlphaStringImage(const ICoord *isize, const Coord *size);
  ~AlphaStringImage();
  const ICoord &sizeInPixels() const { return isize_; }
  const Coord &size() const { return size_; }
  void set(const ICoord *where, const CColor *color, unsigned char alpha);
  const unsigned char *getString() const { return data; }
};

#endif // STRINGIMAGE
