// -*- C++ -*-
// $RCSfile: ooferror.C,v $
// $Revision: 1.11 $
// $Author: langer $
// $Date: 2004/06/04 20:57:26 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>
#include "ooferror.h"
#include "tostring.h"
#include "trace.h"



ErrProgrammingError::ErrProgrammingError(const std::string &f, int l)
  : msg(""), file(f), line(l)
{
}

ErrProgrammingError::ErrProgrammingError(const std::string &m, const std::string &f, int l)
  : msg(m), file(f), line(l)
{
}

std::string ErrProgrammingError::message() const {
  return msg;
//   if(msg.empty())
//     return "In " + file + ", line " + to_string(line) + ": ";
//   else
//     return "In " + file + ", line " + to_string(line) + ": " + msg + " ";
}

std::string ErrResourceShortage::message() const {
  return msg;
}

std::string ErrBadIndex::message() const {
  return ErrProgrammingError::message() + "Bad Index: " + to_string(badindex);
}

//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//
// Python equivalents 

// Makes the most general assumption about the structure.
std::string ErrProgrammingError::pythonequiv() const {
  return "ErrProgrammingError('" + msg + "', '" + file + "', " +
    to_string(line) + ")";
}

std::string IndexError::pythonequiv() const {
  return "IndexError";
}

std::string ErrResourceShortage::pythonequiv() const {
  return "ErrResourceShortage('" + msg + "')";
}

std::string ErrBadIndex::pythonequiv() const {
  return "ErrBadIndex('" + ErrProgrammingError::message() + "')";
}

std::string ErrUserError::pythonequiv() const {
  return "ErrUserError('" + msg + "')";
}

std::string ErrSetupError::pythonequiv() const {
  return "ErrSetupError('" + msg + "')";
}

//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//

ErrNoCanvas::ErrNoCanvas() {
}

ErrNoCanvas::~ErrNoCanvas() {
}

std::string ErrNoCanvas::pythonequiv() const {
  return "ErrNoCanvas()";
}

//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//

/* Mandatory static initializer for the pointer to the Python
   executable code.  Gets properly initialized by in-lined
   Python code when oofcpp is imported. */
PyObject *ErrError::pyconverter(0);

void pyErrorInit(PyObject *converter) {
  Py_XINCREF(converter);
  ErrError::pyconverter = converter;
}

//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//

void testerror() {
  Trace("testerror");
  throw ErrProgrammingError(__FILE__, __LINE__);
}
