# -*- python -*-
# $RCSfile: subthread.py,v $
# $Revision: 1.21 $
# $Author: langer $
# $Date: 2005/07/25 14:34:27 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

## Routines for submitting functions to be run in a subthread.
## When threads are enabled, a thread is launched executing the proposed
## function.If threads are not enabled, the function is readily applied
## on the main thread.

import exceptions
import threading
from oof2.SWIG.common import lock
from oof2.SWIG.common import switchboard
from oof2.SWIG.common.IO import threadstate
from oof2.common import debug
from oof2.common import thread_enable
from oof2.common.IO import reporter


class StopThread (exceptions.Exception):
    def __init__ (self):
        exceptions.Exception.__init__(self)
    

class MiniThread (threading.Thread):
    def __init__(self, function, args=(), kwargs={}):
        self.function = function
        self.args = args
        self.kwargs = kwargs
        threading.Thread.__init__(self)
        self.threadstate = None
        threadstate.cleanList()

    def run(self):
        miniThreadManager.add(self)
        try:
            try:
                self.threadstate = threadstate.findThreadState()
                self.function(*self.args, **self.kwargs)
            except StopThread:
                ## raise StopThread
                return
            except Exception, exception:
                reporter.error(exception)
        finally:
            miniThreadManager.remove(self)
            self.threadstate.make_defunct()

    def stop_it(self):
        threadstate.cancelThread(self.threadstate)

def execute(function, args=(), kwargs={}):
    if thread_enable.query():
        littlethread = MiniThread(function, args, kwargs)
        littlethread.start()
##        miniThreadManager.remove(littlethread)
        return
    function( *args, **kwargs)



## The purpose of the MiniThreadManager is to administer the running (mini)
## threads, so that when quiting time comes all minithreads are asked to stop
## properly, releasing their locks, and freeing the main thread. The main
## thread is left to execute other quiting tasks.

class MiniThreadManager:
    def __init__(self):
        self.listofminithreads =[]
        self.lock = lock.Lock()

    def add(self, minithread):
        self.lock.acquire()
        self.listofminithreads.append(minithread)
        self.lock.release()

    def remove(self,minithread):
        self.lock.acquire()
        self.listofminithreads.remove(minithread)
        self.lock.release()

    def stopAll(self):
        self.lock.acquire()
        threadlist = self.listofminithreads[:]
        self.lock.release()
        for minithread in threadlist:
            minithread.stop_it()
            
    def WaitForAllThreads(self):
        self.lock.acquire()
        threadlist = self.listofminithreads[:]
        self.lock.release()
        for minithread in threadlist:
            minithread.join()

    # Return the calling thread's MiniThread object, if it has one, or
    # None.
    def getMiniThread(self):
        callers_ts = threadstate.findThreadState()
        self.lock.acquire()
        try:
            for mini in self.listofminithreads:
                if mini.threadstate == callers_ts:
                    return mini
        finally:
            self.lock.release()
        
    def quit(self):
        if len(self.listofminithreads)>0:
            self.stopAll()
            
        
miniThreadManager = MiniThreadManager()
##import atexit
##atexit.register(miniThreadManager.quit) ## automatically stop any running mini thread that was left behind at exit.

