# -*- python -*-
# $RCSfile: meshPage.py,v $
# $Revision: 1.76 $
# $Author: langer $
# $Date: 2005/09/07 21:31:56 $


# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.SWIG.common import switchboard
from oof2.SWIG.engine import masterelement
from oof2.common import debug
from oof2.common import guitop
from oof2.common import labeltree
from oof2.common import microstructure
from oof2.common.IO import mainmenu
from oof2.common.IO import parameter
from oof2.common.IO import reporter
from oof2.common.IO.GUI import chooser
from oof2.common.IO.GUI import historian
from oof2.common.IO.GUI import oofGUI
from oof2.common.IO.GUI import parameterwidgets
from oof2.common.IO.GUI import regclassfactory
from oof2.common.IO.GUI import whowidget
from oof2.engine import meshmod
from oof2.engine import skeletoncontext
import gtk
import oof2.engine.mesh
import string

meshmenu = mainmenu.OOF.Mesh


class MeshPage(oofGUI.MainPage):
    def __init__(self):
        oofGUI.MainPage.__init__(self, name="FE Mesh", ordering=200,
                                 tip="Create a Finite Element Mesh from a Skeleton.")
        mainbox = gtk.VBox(spacing=2)
        self.gtk.add(mainbox)

        align = gtk.Alignment(xalign=0.5)
        mainbox.pack_start(align, expand=0, fill=0)
        centerbox = gtk.HBox(spacing=3)
        align.add(centerbox)
        self.meshwidget = whowidget.WhoWidget(oof2.engine.mesh.meshes,
                                              callback=self.meshCB,
                                              scope=self)
        label = gtk.Label("Microstructure=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.meshwidget.gtk[0], expand=0, fill=0)

        label = gtk.Label("Skeleton=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.meshwidget.gtk[1], expand=0, fill=0)

        label = gtk.Label("Mesh=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.meshwidget.gtk[2], expand=0, fill=0)

        # Centered box of buttons
        align = gtk.Alignment(xalign=0.5)
        mainbox.pack_start(align, expand=0, fill=0)
        bbox = gtk.HBox(homogeneous=1, spacing=3)
        align.add(bbox)
        self.newbutton = gtk.Button("New...")
        self.newbutton.connect('clicked', self.newCB)
        guitop.top().tooltip(self.newbutton, "Create a new mesh from the current skeleton.")
        bbox.pack_start(self.newbutton, expand=0, fill=1)
        
        self.renamebutton = gtk.Button("Rename...")
        self.renamebutton.connect('clicked', self.renameCB)
        guitop.top().tooltip(self.renamebutton, "Rename the current mesh.")   
        bbox.pack_start(self.renamebutton, expand=0, fill=1)
        
        self.copybutton = gtk.Button("Copy...")
        self.copybutton.connect('clicked', self.copyCB)
        guitop.top().tooltip(self.copybutton, "Copy the current mesh.")
        bbox.pack_start(self.copybutton, expand=0, fill=1)
        
        self.deletebutton = gtk.Button("Delete")
        self.deletebutton.connect('clicked', self.deleteCB)
        guitop.top().tooltip(self.deletebutton, "Delete the current mesh.")
        bbox.pack_start(self.deletebutton, expand=0, fill=1)
        
        self.savebutton = gtk.Button("Save...")
        self.savebutton.connect('clicked', self.saveCB)
        guitop.top().tooltip(self.savebutton,
                             "Save the current mesh to a file.")
        bbox.pack_start(self.savebutton, expand=0, fill=1)

        mainpane = gtk.HPaned()
        mainbox.pack_start(mainpane, expand=1, fill=1)
        
        infoframe = gtk.Frame('Mesh Information')
        infoframe.set_shadow_type(gtk.SHADOW_IN)
        mainpane.pack1(infoframe, resize=1, shrink=0)
        scroll = gtk.ScrolledWindow()
        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scroll.set_shadow_type(gtk.SHADOW_IN)
        infoframe.add(scroll)
        self.infoarea = gtk.TextView()
        self.infoarea.set_cursor_visible(False)
        self.infoarea.set_editable(False)
        scroll.add(self.infoarea)

        # Right hand side for element operations
        
        elementopsframe = gtk.Frame(label="Element Operations")
        elementopsframe.set_shadow_type(gtk.SHADOW_IN)
        mainpane.pack2(elementopsframe, resize=0, shrink=0)
        elementopsbox = gtk.VBox(spacing=3)
        elementopsframe.add(elementopsbox)
        self.elementops = regclassfactory.RegisteredClassFactory(
            meshmod.MeshModification.registry,
            title="Method:",
            callback=self.elementopsCB,
            expand=0, fill=0, scope=self)
        elementopsbox.pack_start(self.elementops.gtk, expand=1, fill=1)

        self.historian = historian.Historian(self.elementops.set,
                                             self.sensitizeHistory,
                                             setCBkwargs={'interactive':1})
        # Prev, OK, Next
        hbox = gtk.HBox()
        elementopsbox.pack_start(hbox, expand=0, fill=0, padding=2)
        self.prevbutton = gtk.Button()
        lbox = gtk.HBox()
        lbox.pack_start(gtk.Arrow(gtk.ARROW_LEFT, gtk.SHADOW_OUT),
                        expand=0, fill=0)
        lbox.pack_start(gtk.Label(' Prev'), expand=0, fill=0)
        self.prevbutton.add(lbox)
        self.prevbutton.connect('clicked', self.prevCB)
        guitop.top().tooltip(self.prevbutton,
                             "Recall the previous mesh element operation.")
        hbox.pack_start(self.prevbutton, expand=0, fill=0, padding=2)

        self.okbutton = gtk.Button('OK')
        self.okbutton.connect('clicked', self.okCB)
        guitop.top().tooltip(self.okbutton,
                          'Perform the mesh element operation defined above.')
        hbox.pack_start(self.okbutton, expand=1, fill=1, padding=5)

        self.nextbutton = gtk.Button()
        lbox = gtk.HBox()
        lbox.pack_start(gtk.Label('Next '), expand=0, fill=0)
        lbox.pack_start(gtk.Arrow(gtk.ARROW_RIGHT, gtk.SHADOW_OUT),
                        expand=0, fill=0)
        self.nextbutton.add(
            lbox)
        self.nextbutton.connect('clicked', self.nextCB)
        guitop.top().tooltip(self.nextbutton,
                             'Recall the next mesh element operation.')
        hbox.pack_start(self.nextbutton, expand=0, fill=0, padding=2)
        

        switchboard.requestCallback("Mesh modified", self.recordModifier)
        switchboard.requestCallback(("new who", "Skeleton"), self.newSkeleton)
        switchboard.requestCallback(("new who", "Mesh"), self.newMesh)
        switchboard.requestCallback(("remove who", "Mesh"), self.removeMesh)
        switchboard.requestCallback("equation activated", self.equationCB)
        switchboard.requestCallback("mesh boundaries changed",
                                    self.newMeshBoundaries)

        switchboard.requestCallback(('validity', self.elementops),
                                    self.validityChangeCB)
        
        self.sensitize()
        self.sensitizeHistory()

    def show(self):
        debug.mainthreadTest()
        self.gtk.show_all()
        self.update()

    def newMeshBoundaries(self, mesh):
        if mesh==self.currentMesh():
            self.update()
            self.sensitize()
            
    #######################

    def currentMSName(self):
        path = labeltree.makePath(self.currentFullMeshName())
        return path[0]
    def currentMS(self):
        msname = self.currentMSName()
        if msname:
            return microstructure.microStructures[msname].getObject()

    def currentSkeletonPath(self):
        path = labeltree.makePath(self.currentFullMeshName())
        return path[0:2]
    def currentSkeletonFullName(self):
        return string.join(self.currentSkeletonPath(), ":")
    def currentSkeleton(self):
        ms = self.currentMS()
        skelname = self.currentSkeletonPath()[-1]
        if skelname:
            return skeletoncontext.getSkeleton(ms, skelname)

    def currentFullMeshName(self):
        return self.meshwidget.get_value()
    def currentMeshName(self):
        path = labeltree.makePath(self.currentFullMeshName())
        return path[2]
    def currentMesh(self):
        try:
            return oof2.engine.mesh.meshes[self.currentFullMeshName()].getObject()
        except KeyError:
            return None
            
    def sensitize(self):
        debug.mainthreadTest()
        msok = self.currentMS() is not None
        skelok = msok and self.currentSkeleton() is not None
        meshok = skelok and self.currentMesh() is not None
        self.newbutton.set_sensitive(skelok)
        self.deletebutton.set_sensitive(meshok)
        self.renamebutton.set_sensitive(meshok)
        self.copybutton.set_sensitive(meshok)
        self.savebutton.set_sensitive(meshok)
        self.okbutton.set_sensitive(meshok and self.elementops.isValid())

    def sensitizeHistory(self):
        debug.mainthreadTest()
        self.nextbutton.set_sensitive(self.historian.nextSensitive())
        self.prevbutton.set_sensitive(self.historian.prevSensitive())
        
    def update(self):
        self.set_state(self.currentFullMeshName())

    def recordModifier(self, path, modifier):  # callback for "Mesh modified"
        if modifier:
            self.historian.record(modifier)
        # might as well do the update for Mesh Information pane.
        self.set_state(path)
        
    def set_state(self, meshpath):  # widget update & information update
        debug.mainthreadTest()
        path = labeltree.makePath(meshpath)
        self.meshwidget.set_value(path)
        buffer = self.infoarea.get_buffer()
        buffer.delete(buffer.get_start_iter(), buffer.get_end_iter())
        if path[-1]:                    # ie, mesh name is not empty
            themesh = oof2.engine.mesh.meshes[path]
            skel = themesh.getSkeleton()
            buffer.insert(buffer.get_end_iter(),
                          "No. of Nodes:\t%d\n" % themesh.nnodes())
            buffer.insert(buffer.get_end_iter(),
                          "No. of Elements:\t%d\n" % themesh.nelements())
            masterelementenums = masterelement.getMasterElementEnums()
            sides = masterelementenums.keys() # list of element geometries
            sides.sort()
            counts = [0]*(max(sides)+1)
            for elem in skel.element_iterator():
                counts[elem.nnodes()] += 1
            for nsides in sides:
                buffer.insert(buffer.get_end_iter(),
                              "%d sided element:\t%s (%d)\n"
                              % (nsides, themesh.getMasterElementType(nsides),
                                 counts[nsides]))
            
##            for nsides in sides:
##                count = 0
##                for elem in skel.element_iterator():
##                    if elem.nnodes() == nsides:
##                        count += 1
##                self.infolist.append([`nsides` +" sided element:",
##                                      themesh.getMasterElementType(nsides)+" ("+`count`+")"])
##                self.infolist.set_selectable(row, 0)
##                row += 1
            
        else:                           # no current mesh
            buffer.insert(buffer.get_end_iter(), "No mesh!")


        self.sensitize()
        self.sensitizeHistory()

    def newSkeleton(self, skeletonname): # switchboard ("new who", "Skeleton")
        self.sensitize()

    def newMesh(self, meshname):        # switchboard ("new who", "Mesh")
        self.set_state(meshname)
        self.sensitize()
        
    def meshCB(self, meshpath):         # WhoWidget callback
        self.set_state(meshpath)
        self.sensitize()

    def equationCB(self, *args):  # switchboard "equation activated"
        switchboard.notify(self.meshwidget, interactive=1)

    def newCB(self, *args):             # gtk button callback
        menuitem = mainmenu.OOF.Mesh.New
        params = filter(lambda x: x.name !='skeleton', menuitem.params)
        if parameterwidgets.getParameters(title='Create a new mesh', scope=self, *params):
            menuitem.callWithDefaults(skeleton=self.currentSkeletonFullName())

    def deleteCB(self, *args):          # gtk button callback
        if reporter.query(
            "Really delete %s?"%self.currentFullMeshName(),
            "No", default="Yes") == "Yes":
            meshmenu.Delete(mesh=self.currentFullMeshName())

    def removeMesh(self, path):    # switchboard ("remove who", "Mesh")
        self.update()
        self.sensitize()

    def copyCB(self, *args):            # gtk button callback
        menuitem = mainmenu.OOF.Mesh.Copy
        nameparam = menuitem.get_arg("name")
        fieldparam = menuitem.get_arg("copy_field")
        eqnparam = menuitem.get_arg("copy_equation")
        bcparam = menuitem.get_arg("copy_bc")
        if parameterwidgets.getParameters(nameparam, fieldparam, eqnparam,
                                          bcparam, title='Copy a mesh'):
            menuitem.callWithDefaults(mesh=self.currentFullMeshName())
        
    def renameCB(self, *args):          # gtk button callback
        menuitem = meshmenu.Rename
        namearg = menuitem.get_arg('name')
        curmeshpath = self.currentFullMeshName()
        namearg.value = labeltree.makePath(curmeshpath)[-1]
        if parameterwidgets.getParameters(namearg,
                                          title='Rename mesh '+namearg.value):
            menuitem.callWithDefaults(mesh=curmeshpath)

    def saveCB(self, *args):
        menuitem = mainmenu.OOF.File.Save.Mesh
        meshname = self.meshwidget.get_value()
        file = fileselector.fileSelector(
            ident='SaveMeshFromPage',
            title='Save Mesh "%s"' % meshname,
            params=[menuitem.get_arg('format')])
        if file is not None:
            mode = fileselector.get_file_mode(file)
            if mode is not None:
                menuitem.callWithDefaults(filename=file, mode=mode,
                                          mesh=meshname)
    def prevCB(self, gtkobj):
        self.historian.prevCB()

    def nextCB(self, gtkobj):
        self.historian.nextCB()

    def elementopsCB(self, reg):
        self.historian.stateChangeCB(reg)
        self.sensitize()

    def okCB(self, gtkobj):
        path = self.meshwidget.get_value()
        modifier = self.elementops.get_value()
        if path and modifier:
            mainmenu.OOF.Mesh.Modify(mesh=path, modifier=modifier)

    def validityChangeCB(self, validity):
        self.sensitize()
        

#############
        
mp = MeshPage()

###########################################################

from oof2.common.IO.GUI import fileselector

def _save_mesh(menuitem):
    if len(oof2.engine.mesh.meshes) > 0:
        file = fileselector.fileSelector(
            ident='SaveMeshFromMenu',
            title='Save Mesh',
            params=[menuitem.get_arg('mesh'), menuitem.get_arg('format')]
            )
        if not file is None:
            mode = fileselector.get_file_mode(file)
            if mode is not None:
                menuitem.callWithDefaults(filename=file, mode=mode)
    else:
        reporter.report("No mesh?")

mainmenu.OOF.File.Save.Mesh.add_gui_callback(_save_mesh)
