# -*- python -*-
# $RCSfile: instantnodemove.py,v $
# $Revision: 1.7 $
# $Author: langer $
# $Date: 2005/05/13 18:44:05 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

from oof2.common import registeredclass
from oof2.common.IO import parameter
from oof2.engine import skeletonmodifier
from oof2.common import utils
from oof2.common import primitives
import types
import math

SkeletonModifier = skeletonmodifier.SkeletonModifier
FloatRangeParameter = parameter.FloatRangeParameter

class InstantNodemove(SkeletonModifier):
    def __init__(self,alpha=0.5):
        self.alpha = alpha
        self.selectedNode = None
        self.clickedPosition = None

    def execute(self,skeleton,point):
        self.clickedPosition = point
        count = 0
        for nd in skeleton.nodes:
            if count>1:
                print "Can't move two nodes at a time!"
                return skeleton
            if nd.selected:
                count += 1
                self.selectedNode = nd

        if count==1:
            if self.selectedNode.movability!=0:
                if self.moveTheNode(skeleton):
                    print "The selected node moved to the clicked position."
                else:
                    print "The desired move doesn't help the situation!"
            else:
                print "The selected node cannot move."

        return skeleton
            
    def moveTheNode(self,skeleton):
        neighborList = self.selectedNode.neighborElements()
        backup = primitives.Point(self.selectedNode.position.x,
                                  self.selectedNode.position.y)
        ratio = []; dom=[]; contri=[]; energy=[]; area=0.0
        for e in neighborList:
            ratio.append(e.homogeneity)
            dom.append(e.dominant)
            contri.append(e.contribution)
            energy.append(self.alpha*e.energyHomo()+\
                          (1.0-self.alpha)*e.energyShape())
            area += e.area()
        
        if self.tryThisMove(neighborList,skeleton,backup,ratio,
           dom,contri,energy,area):
            return 1
        else:
            return 0

    def tryThisMove(self,neighborList,skeleton,backup,ratio,
                    dom,contri,energy,area):
        if self.disqualifyDisp(self.clickedPosition,neighborList,skeleton):
            return 0

        if self.selectedNode.movability==1:
            self.selectedNode.position.x = self.clickedPosition.x
        elif self.selectedNode.movability==2:
            self.selectedNode.position.y = self.clickedPosition.y
        else:
            self.selectedNode.position = self.clickedPosition

        for e in neighborList:
            e.getHomogeneity(skeleton)

        preEnergy = 0.0
        newEnergy = 0.0
        newArea = 0.0
        tolerence = \
            skeleton.MS.sizeOfPixels()[0]*skeleton.MS.sizeOfPixels()[1]/1.0e9
        for i in range(len(neighborList)):
            e = neighborList[i]
            preEnergy += energy[i]
            newEnergy += self.alpha*e.energyHomo()+(1.0-self.alpha)*e.energyShape()
            newArea += e.area()
        if preEnergy<newEnergy or abs(area-newArea)>tolerence:
            self.backwardMove(self.selectedNode,backup,ratio,dom,contri,
                              neighborList)
            return 0
        else: return 1

    def disqualifyDisp(self,newPosition,neighborList,skeleton):
        atLeastOne = 0
        for e in neighborList:
            if e.interior(newPosition): atLeastOne = 1
        if not atLeastOne: return 1

        return 0

    def backwardMove(self,node,backup,ratio,dom,contri,neighborList):
        # restores original node position and status
        node.position=backup
        for i in range(len(neighborList)):
            neighborList[i].homogeneity=ratio[i]
            neighborList[i].dominant=dom[i]
            neighborList[i].contribution = contri[i]

    def apply(self,skeleton):
        return skeleton.instantNodemove(self)

registeredclass.Registration('InstantNodemove',SkeletonModifier,
                             InstantNodemove,0,
                             params=\
                             [FloatRangeParameter('alpha',(0.0,1.0,0.01),value=0.5,
                              tip='E = alpha E_homo + (1-alpha) E_shape')],
                             tip='Move a selected node to a clicked position.')

